function spm_denoise()
% plots Figure 4 of paper "RECENT ADVANCES IN TIME-FREQUENCY REASSIGNMENT AND
%   SYNCHROSQUEEZING", by Meignen et al.


% Signal
N = 1024;
T = 1;
t = (0:N-1)/N;
un = ones(size(t));

a = exp(-8*(t-T/2*un).^2);
s1 = a.*sin(2*pi*(250*t+100*t.^3));
s2 = a.*sin(2*pi*(130*t+100*t.^2));
s3 = a.*sin(2*pi*(90*t+0.2*cos(3*pi*t)));
so = s1+s2+s3;
so = so(:);

% normalize
so = so/norm(so)*sqrt(length(so));

% Constants
N = length(so);
fs = N;
frecv = 0:N/2-1;


% Parameters
lambda = 0.02;
gamma = 0.00001;
index = N/8:7*N/8;
sigma1 = 0.03;
sigma2 = 0.06;
sigma3 = 0.1;
nmodes = 3;
del = 8;


%% Iteration for different noise values
snr = -5:1:30; % levels of noise (snr)
P = length(snr);
rep = 20; % Number of realizations for each noise level
rests1 = zeros(1,P); % SST reconstruction sigma 1
rests2 = zeros(1,P); % SST reconstruction sigma 2
rests3 = zeros(1,P); % SST reconstruction sigma 3
resbt = zeros(1,P); % Block thresholding 

for k=1:P
    for l=1:rep
        sig = exp(-snr(k)/20)/sqrt(length(index)/N); % sigma
        % The SNR is computed only for coefficients in index, hence the
        % corrective term 1//sqrt(length(index)/N)
        
        s = so+sig*randn(size(so));
        
        % SST and ridge extraction sigma 1
        %[Vs Ts Rs] = sst(s,gamma,sigma1);
        [Vs Ts] = fvst(s,gamma,sigma1,1);
        
        % Ridge extraction from Ts
        [Cs, Es] = brevridge_mult(Ts, frecv, nmodes, lambda, 25);
        srec = zeros(size(s));
        for kl=1:N
            for jl=1:nmodes
                srec(kl) = srec(kl) + sum(Ts(max(1,Cs(jl,kl)-del):min(Cs(jl,kl)+del,N/2),kl));
            end
        end
        srec = 2*real(srec);
        rests1(k) = rests1(k) -20*log(norm(so(index)-srec(index))/norm(so(index)));
        

        % SST and ridge extraction sigma 2
        %[Vs Ts Rs] = sst(s,gamma,sigma2);
        [Vs Ts] = fvst(s,gamma,sigma2,1);
        
        % Ridge extraction from Ts
        [Cs, Es] = brevridge_mult(Ts, frecv, nmodes, lambda, 25);
        srec = zeros(size(s));
        for kl=1:N
            for jl=1:nmodes
                srec(kl) = srec(kl) + sum(Ts(max(1,Cs(jl,kl)-del):min(Cs(jl,kl)+del,N/2),kl));
            end
        end
        srec = 2*real(srec);
        rests2(k) = rests2(k) -20*log(norm(so(index)-srec(index))/norm(so(index)));

        
        % SST and ridge extraction sigma 3
        %[Vs Ts Rs] = sst(s,gamma,sigma3);
        [Vs Ts] = fvst(s,gamma,sigma3,1);
        
        % Ridge extraction from Ts
        [Cs, Es] = brevridge_mult(Ts, frecv, nmodes, lambda, 25);
        srec = zeros(size(s));
        for kl=1:N
            for jl=1:nmodes
                srec(kl) = srec(kl) + sum(Ts(max(1,Cs(jl,kl)-del):min(Cs(jl,kl)+del,N/2),kl));
            end
        end
        srec = 2*real(srec);
        rests3(k) = rests3(k) -20*log(norm(so(index)-srec(index))/norm(so(index)));

        % Blockthresholding
        [srec, AttenFactorMap, flag_depth] = BlockThresholding(s, 20, N, sig);
        resbt(k) = resbt(k) -20*log(norm(so(index)-srec(index)')/norm(so(index)));
        
    end
end

rests1 = rests1/rep;
rests2 = rests2/rep;
rests3 = rests3/rep;
resbt = resbt/rep;

% Display
figure();
plot(snr,rests1,'b',snr,rests2,'k-.',snr,rests3,'m:',snr,resbt,'r--');
legend('SST \sigma=0.03','SST \sigma=0.06','SST \sigma=0.10','BT');
xlabel('SNR in (dB)');
ylabel('SNR out (dB)');
explot();
set(findall(0,'type','Line'),'LineWidth',2);



end

