function spm_fig1()
%SPM_FIG1: plots Figure 2 of paper "RECENT ADVANCES IN TIME-FREQUENCY REASSIGNMENT AND
%   SYNCHROSQUEEZING", by Meignen et al.


N = 4*1024;
T = 1;
t = (0:N-1)/N;
un = ones(size(t));

%% Multicomponent signal

a = exp(-8*(t-T/2*un).^2);
s1 = a.*sin(2*pi*(250*t+100*t.^3));
s2 = a.*sin(2*pi*(130*t+100*t.^2));
s3 = a.*sin(2*pi*(90*t+0.2*cos(3*pi*t)));

s = s1+s2+s3;


N = length(s);
dt = 1/N;
fs = N;
frecv = 1:512;
timev = 1:N;

s = s/norm(s)*sqrt(N);

% Parameters
sigma=0.035;
lambda = 0.02;
gamma = 0.05;

[Vs Ts Rs] = sst(s,gamma,sigma,frecv,timev);

% zoom coordinates
xn=0.5;xN=0.8;
yn=320/2/pi;yN=450/2/pi;

%% Display

% STFT
ha = subplot(2,4,[1 5]);
tfplot(Vs,t(timev),(frecv-1)*fs/N/2/pi,ha);
rectangle('Position',[xn yn xN-xn yN-yn],'EdgeColor','red','Linewidth',2);

% Zoom
ha = subplot(2,4,[2 6]);
tfplot(Vs,t(timev),(frecv-1)*fs/N/2/pi,ha);set(ha,'xlim',[xn xN],'ylim',[yn yN]);
set(ha,'xtick',[],'ytick',[]);xlabel('');ylabel('');

% RM
ha = subplot(2,4,3);
tfplot(Rs,t(timev),(frecv-1)*fs/N/2/pi,ha);set(ha,'xlim',[xn xN],'ylim',[yn yN]);
set(ha,'xtick',[],'ytick',[]);xlabel('');ylabel('');

% SST
ha = subplot(2,4,7);
tfplot(Ts,t(timev),(frecv-1)*fs/N/2/pi,ha);set(ha,'xlim',[xn xN],'ylim',[yn yN]);
set(ha,'xtick',[],'ytick',[]);xlabel('');ylabel('');


end

